//
//  ICMainVC.m
//  iosCalibrateDemo
//
//  Created by waytronic on 2024/9/10.
//

#import "ICMainVC.h"
#import "ICPeripheralVC.h"
#import "ICDeviceListCell.h"
#import "ICDeviceNoneView.h"
#import "MJRefresh.h"

@interface ICMainVC ()<UITableViewDelegate,UITableViewDataSource>

@property(nonatomic,strong) UITableView *tableView;

@property(nonatomic,strong) NSMutableArray<ICDeviceModel *> *deviceList;

@end


@interface ICMainVC ()

@end

@implementation ICMainVC

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.navigationItem.title = @"搜索设备";
    
    [self.view addSubview:self.tableView];
    MJRefreshNormalHeader *header = [MJRefreshNormalHeader headerWithRefreshingTarget:self refreshingAction:@selector(pullDownRefresh)];
    self.tableView.mj_header = header;
    
    [[ICBleManager shared] initService];
    [ICBleManager shared].deviceFoundBlock = ^(NSArray<ICDeviceModel *> *modelList) {
        self.deviceList = modelList.mutableCopy;
        [self.tableView reloadData];
    };
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [self pullDownRefresh];
}

//下拉刷新
- (void)pullDownRefresh
{
    [self.deviceList removeAllObjects];
    [self.tableView reloadData];
    [[ICBleManager shared] cancelScan];
    [[ICBleManager shared] beginScanPeripherals];
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(.1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        [self.tableView.mj_header endRefreshing];
        [self.tableView reloadData];
    });
}

#pragma mark - getter
- (NSMutableArray<ICDeviceModel *> *)deviceList {
    if (_deviceList == nil)
    {
        _deviceList = [NSMutableArray array];
    }
    return _deviceList;
} 

- (UITableView *)tableView
{
    if (_tableView == nil) {
        _tableView = [[UITableView alloc] initWithFrame:CGRectMake(0, ICNaviHeight, ICWidth, ICHeight - ICNaviHeight) style:UITableViewStylePlain];
        _tableView.delegate = self;
        _tableView.dataSource = self;
        _tableView.separatorStyle = UITableViewCellSeparatorStyleNone;
        [_tableView registerClass:ICDeviceListCell.class forCellReuseIdentifier:ICDeviceListCell.ID];
    }
    return _tableView;
}


#pragma mark - tableView
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    tableView.backgroundView = self.deviceList.count == 0 ? [ICDeviceNoneView new] : nil;
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section{
    return self.deviceList.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    ICDeviceListCell *cell = [tableView dequeueReusableCellWithIdentifier:ICDeviceListCell.ID forIndexPath:indexPath];
    cell.deviceModel = self.deviceList[indexPath.row];
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 60;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    [[ICBleManager shared] cancelScan];
    
    ICPeripheralVC *peripheralVC = [[ICPeripheralVC alloc] init];
    peripheralVC.currPeripheral = self.deviceList[indexPath.row].peripheral;
    [self.navigationController pushViewController:peripheralVC animated:YES];
}

@end
