//
//  ICPeripheralVC.m
//  iosCalibrateDemo
//
//  Created by waytronic on 2024/9/11.
//

#import "ICPeripheralVC.h"
#import "ICToast.h"

@interface ICPeripheralVC ()<ICLocationUtilsDelegate,ICBleConnectDelegate>

//定位服务
@property(nonatomic,strong) ICLocationUtils *locationUtils;

//wifi名称
@property(nonatomic,strong) UITextField *wifiNameTF;

//密码
@property(nonatomic,strong) UITextField *passwordTF;

@end

@implementation ICPeripheralVC

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.navigationItem.title = @"配网";
    
    /** 设置代理 */
    [ICBleManager shared].connectDelegate = self;
    
    /** 连接设备  */
    [[ICBleManager shared] connectPeripheral:self.currPeripheral];
    
    /** 构建UI  */
    [self setupUI];
}

- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event {
    [self.view endEditing:YES];
}

//设置UI
- (void)setupUI
{
    WEAkSELF(self);
    
    self.view.backgroundColor = [UIColor whiteColor];
    
    self.wifiNameTF = self.view
        .addTextField(1)
        .font([UIFont systemFontOfSize:16])
        .clearButtonMode(UITextFieldViewModeWhileEditing)
        .attributedPlaceholder([[NSMutableAttributedString alloc] initWithString:@"请输入Wifi名称" attributes:@{NSFontAttributeName:[UIFont systemFontOfSize:16],NSForegroundColorAttributeName:UIColorFromRGB(0x999999)}])
        .masonry(^(__kindof UIView *sender, MASConstraintMaker *make) {
            make.left.equalTo(self.view).offset(20);
            make.centerX.equalTo(self.view);
            make.top.equalTo(self.view).offset(ICNaviHeight + 30);
            make.height.mas_equalTo(40);
        })
        .view;
    
   UIView *wifiNameBottomLineV = self.view.addView(2)
        .backgroundColor(UIColorFromRGB(0xF5F5F5))
        .masonry(^(__kindof UIView *sender, MASConstraintMaker *make) {
            make.top.equalTo(self.wifiNameTF.mas_bottom);
            make.left.right.equalTo(self.wifiNameTF);
            make.height.mas_equalTo(1);
    }).view;
    
    
    self.passwordTF = self.view
        .addTextField(1)
        .font([UIFont systemFontOfSize:16])
        .clearButtonMode(UITextFieldViewModeWhileEditing)
        .attributedPlaceholder([[NSMutableAttributedString alloc] initWithString:@"请输入Wifi密码" attributes:@{NSFontAttributeName:[UIFont systemFontOfSize:16],NSForegroundColorAttributeName:UIColorFromRGB(0x999999)}])
        .masonry(^(__kindof UIView *sender, MASConstraintMaker *make) {
            make.top.equalTo(wifiNameBottomLineV.mas_bottom).offset(20);
            make.left.right.height.equalTo(self.wifiNameTF);
        })
        .view;
    self.passwordTF.autocorrectionType = UITextAutocorrectionTypeNo;
    
    
    UIView *passwordBottomLineV = self.view
        .addView(3)
        .backgroundColor(UIColorFromRGB(0xF5F5F5))
        .masonry(^(__kindof UIView *sender, MASConstraintMaker *make) {
            make.top.equalTo(self.passwordTF.mas_bottom);
            make.left.right.height.equalTo(wifiNameBottomLineV);
    }).view;
    
    self.view.addButton(3)
        .backgroundColor(UIColorFromRGB(0x573AFD))
        .title(@"开始配网")
        .cornerRadius(10)
        .eventTouchUpInside(^(__kindof UIControl *sender) {
            [weakself startConfigNetwork];
        })
        .masonry(^(__kindof UIView *sender, MASConstraintMaker *make) {
            make.top.equalTo(passwordBottomLineV.mas_bottom).offset(30);
            make.centerX.equalTo(self.view);
            make.size.mas_equalTo(CGSizeMake(285, 50));
        });
    

    [self.locationUtils requireWifiName];

}


//开始配网
- (void)startConfigNetwork
{
    [WSProgressHUD show];
    NSString *wifiName = self.wifiNameTF.text;
    if([ICToolUtils isEmptyText:wifiName])
    {
        [WSProgressHUD dismiss];
        [ICToast showWithText:@"请输入Wifi名称"];
        return;
    }
    NSString *password = self.passwordTF.text;
    if([ICToolUtils isEmptyText:password])
    {
        [WSProgressHUD dismiss];
        [ICToast showWithText:@"请输入Wifi密码"];
        return;
    }
    
    [self.view endEditing:YES];
    
    
    [[ICBleManager shared] startConfigNetwork:wifiName password:password];
    
    
}

/** 统一回到首界面 */
- (void)commonBackMainVC
{
    //断开连接后 需要按键重启设备才能搜到蓝牙
    [self.navigationController popToRootViewControllerAnimated:YES];
}

- (void)dealloc
{
    [[ICBleManager shared] setConnectDelegate:nil];
    [[ICBleManager shared] disconnect];
}

#pragma mark - Getter

- (ICLocationUtils *)locationUtils
{
    if(_locationUtils == nil)
    {
        _locationUtils = [[ICLocationUtils alloc] init];
        _locationUtils.delegate = self;
    }
    return _locationUtils;
}

#pragma mark - ICLocationUtilsDelegate
- (void)receiveWifiName:(NSString *)wifiName
{
    self.wifiNameTF.text = wifiName;
}

#pragma mark - ICBleConnectDelegate
/** 连接失败的回调 */
- (void)onConnectedFail:(CBPeripheral *)peripheral
{
    [WSProgressHUD dismiss];
    [ICToast showWithText:@"连接失败"];
    [self commonBackMainVC];
}

/** 连接断开的回调 */
- (void)onDisconnected:(CBPeripheral *)peripheral
{
    [WSProgressHUD dismiss];
    [ICToast showWithText:@"已断开连接"];
    [self commonBackMainVC];
}

/** 配网结果 */
- (void)onReceiveConfigNetworkResult:(BOOL)isSuccess
{
    [WSProgressHUD dismiss];
    
    if(isSuccess)
    {              
        [self commonBackMainVC];
        [ICToast showWithText:@"配置发送成功,请等待指示灯变化"];
    }
    else
    {
        [ICToast showWithText:@"配网失败"];
    }
}

@end
