//
//  ICBleManager.h
//  WtrOnlineIOS_Example
//
//  Created by waytronic on 2024/9/19.
//  Copyright © 2024 luheng. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <WtrOnlineIOS/ICEnum.h>
#import <WtrOnlineIOS/ICDeviceModel.h>

NS_ASSUME_NONNULL_BEGIN

//搜索设备列表block
typedef void(^ICDeviceFoundBlock)(NSArray<ICDeviceModel *>  * _Nullable modelList);

//蓝牙连接的回调
@protocol ICBleConnectDelegate <NSObject>

@optional
/** 连接成功的回调 */
- (void)onConnectedSuccess:(CBPeripheral *)peripheral;

/** 连接失败的回调 */
- (void)onConnectedFail:(CBPeripheral *)peripheral;

/** 连接断开的回调 */
- (void)onDisconnected:(CBPeripheral *)peripheral;

/** 蓝牙结果回调 原始数据 */
- (void)onReceiveResult:(BOOL)isSuccess data:(struct ICResultData)data;

/** 配网结果 */
- (void)onReceiveConfigNetworkResult:(BOOL)isSuccess;

@end


@interface ICBleManager : NSObject

+ (instancetype)shared;

/** 初始化服务 */
- (void)initService;

/** 设备扫描结果回调 */
@property(nonatomic,copy) ICDeviceFoundBlock _Nullable deviceFoundBlock;

/** 蓝牙连接Delegate  */
@property(nonatomic,weak) id<ICBleConnectDelegate> connectDelegate;

/** 蓝牙连接状态  */
@property(nonatomic,assign,readonly) BOOL isConnected;

/** 开始扫描设备 */
- (void)beginScanPeripherals;

/** 停止扫描 */
- (void)cancelScan;

/** 连接设备 */
- (void)connectPeripheral:(CBPeripheral *)currPeripheral;

/** 断开连接 */
- (void)disconnect;

/** 开始配网 */
- (void)startConfigNetwork:(NSString *)wifiName password:(NSString *)password;

@end

NS_ASSUME_NONNULL_END
